@@ -1,6 +1,5 @@
-#[cfg(feature = "cli")]
-use indicatif::ProgressBar;
-use ndarray::{s, Array1, Array2, ArrayView1, AsArray};
+use crate::*;
+use ndarray::{s, Array1, Array2, AsArray};
 use std::time::Instant;
 
 /// The terminal condition of the algorithm setting.
@@ -16,62 +15,24 @@ pub enum Task {
 }
 
 /// The data of generation sampling.
-#[derive(Copy, Clone)]
+#[derive(Clone)]
 pub struct Report {
+    /// Generation.
     pub gen: u32,
-    pub fitness: f64,
+    /// Best fitness.
+    pub best_f: f64,
+    /// Time duration.
     pub time: f64,
 }
 
-/// The base of the objective function.
-///
-/// For example:
-/// ```
-/// use metaheuristics_nature::ObjFunc;
-/// use ndarray::{AsArray, ArrayView1, Array1};
-/// struct MyFunc(Array1<f64>, Array1<f64>);
-/// impl MyFunc {
-///     fn new() -> Self { Self(Array1::zeros(3), Array1::ones(3) * 50.) }
-/// }
-/// impl ObjFunc for MyFunc {
-///     type Result = f64;
-///     fn fitness<'a, A>(&self, gen: u32, v: A) -> f64
-///     where
-///         A: AsArray<'a, f64>,
-///     {
-///         let v = v.into();
-///         v[0] * v[0] + v[1] * v[1] + v[2] * v[2]
-///     }
-///     fn result<'a, A>(&self, v: A) -> Self::Result
-///     where
-///         A: AsArray<'a, f64>
-///     {
-///         self.fitness(0, v)
-///     }
-///     fn ub(&self) -> ArrayView1<f64> { self.1.view() }
-///     fn lb(&self) -> ArrayView1<f64> { self.0.view() }
-/// }
-/// ```
-/// The objective function returns fitness value that used to evaluate the objective.
-///
-/// The lower bound and upper bound represents the number of variables at the same time.
-///
-/// This trait is designed as immutable.
-pub trait ObjFunc {
-    /// The result type.
-    type Result;
-    /// Return fitness, the smaller value represents good.
-    fn fitness<'a, A>(&self, gen: u32, v: A) -> f64
-    where
-        A: AsArray<'a, f64>;
-    /// Return the final result of the problem.
-    fn result<'a, A>(&self, v: A) -> Self::Result
-    where
-        A: AsArray<'a, f64>;
-    /// Get upper bound.
-    fn ub(&self) -> ArrayView1<f64>;
-    /// Get lower bound.
-    fn lb(&self) -> ArrayView1<f64>;
+impl Report {
+    pub fn new(gen: u32, best_f: f64, time_start: Instant) -> Self {
+        Self {
+            gen,
+            best_f,
+            time: (Instant::now() - time_start).as_secs_f64(),
+        }
+    }
 }
 
 setting_builder! {
@@ -83,45 +44,12 @@ setting_builder! {
     }
 }
 
-#[cfg(feature = "cli")]
-struct PB(Option<ProgressBar>);
-#[cfg(not(feature = "cli"))]
-struct PB;
-
-impl PB {
-    fn new(_len: u32) -> Self {
-        #[cfg(feature = "cli")]
-        if _len > 0 {
-            Self(Some(ProgressBar::new(_len as u64)))
-        } else {
-            Self(None)
-        }
-        #[cfg(not(feature = "cli"))]
-        Self
-    }
-
-    fn inc(&self) {
-        #[cfg(feature = "cli")]
-        if let Some(pb) = &self.0 {
-            pb.inc(1);
-        }
-    }
-
-    fn finish(&self) {
-        #[cfg(feature = "cli")]
-        if let Some(pb) = &self.0 {
-            pb.finish();
-        }
-    }
-}
-
 /// The base class of algorithms.
 /// Please see [`Algorithm`] for more information.
 pub struct AlgorithmBase<F: ObjFunc> {
     pub pop_num: usize,
     pub dim: usize,
     pub gen: u32,
-    pb: PB,
     rpt: u32,
     pub task: Task,
     pub best_f: f64,
@@ -141,15 +69,10 @@ impl<F: ObjFunc> AlgorithmBase<F> {
             assert_eq!(lb.len(), ub.len(), "different dimension of the variables!");
             lb.len()
         };
-        let pb_gen = match settings.task {
-            Task::MaxGen(gen) if cfg!(feature = "cli") => gen,
-            _ => 0,
-        };
         Self {
             pop_num: settings.pop_num,
             dim,
             gen: 0,
-            pb: PB::new(pb_gen),
             rpt: settings.rpt,
             task: settings.task,
             best_f: f64::INFINITY,
@@ -161,17 +84,16 @@ impl<F: ObjFunc> AlgorithmBase<F> {
             func,
         }
     }
+
     /// Get fitness from individual `i`.
     pub fn fitness(&mut self, i: usize) {
         self.fitness[i] = self.func.fitness(self.gen, self.pool.slice(s![i, ..]));
     }
+
     /// Record the performance.
     fn report(&mut self) {
-        self.reports.push(Report {
-            gen: self.gen,
-            fitness: self.best_f,
-            time: (Instant::now() - self.time_start).as_secs_f64(),
-        });
+        self.reports
+            .push(Report::new(self.gen, self.best_f, self.time_start));
     }
 }
 
@@ -204,20 +126,26 @@ impl<F: ObjFunc> AlgorithmBase<F> {
 pub trait Algorithm<F: ObjFunc> {
     /// Return a base handle.
     fn base(&self) -> &AlgorithmBase<F>;
+
     /// Return a mutable base handle.
     fn base_mut(&mut self) -> &mut AlgorithmBase<F>;
+
     /// Initialization implementation.
     fn init(&mut self) {}
+
     /// Processing implementation of each generation.
     fn generation(&mut self);
+
     /// Get lower bound with index.
     fn lb(&self, i: usize) -> f64 {
         self.base().func.lb()[i]
     }
+
     /// Get upper bound with index.
     fn ub(&self, i: usize) -> f64 {
         self.base().func.ub()[i]
     }
+
     /// Assign from source.
     fn assign_from<'a, A>(&mut self, i: usize, f: f64, v: A)
     where
@@ -227,12 +155,14 @@ pub trait Algorithm<F: ObjFunc> {
         b.fitness[i] = f;
         b.pool.slice_mut(s![i, ..]).assign(&v.into());
     }
+
     /// Set the index to best.
     fn set_best(&mut self, i: usize) {
         let b = self.base_mut();
         b.best_f = b.fitness[i];
         b.best.assign(&b.pool.slice(s![i, ..]));
     }
+
     /// Find the best, and set it globally.
     fn find_best(&mut self) {
         let b = self.base_mut();
@@ -246,6 +176,7 @@ pub trait Algorithm<F: ObjFunc> {
             self.set_best(best);
         }
     }
+
     /// Initialize population.
     fn init_pop(&mut self) {
         let mut best = 0;
@@ -262,6 +193,7 @@ pub trait Algorithm<F: ObjFunc> {
             self.set_best(best);
         }
     }
+
     /// Check the bounds.
     fn check(&self, s: usize, v: f64) -> f64 {
         if v > self.ub(s) {
@@ -280,14 +212,19 @@ pub trait Solver<F: ObjFunc>: Algorithm<F> {
     fn history(&self) -> Vec<Report> {
         self.base().reports.clone()
     }
+
     /// Return the x and y of function.
     /// The algorithm must be executed once.
     fn result(&self) -> (Array1<f64>, f64) {
         let b = self.base();
         (b.best.clone(), b.best_f)
     }
+
     /// Start the algorithm and return the final result.
-    fn run(&mut self) -> F::Result {
+    ///
+    /// Support a callback function, such as progress bar.
+    /// To suppress it, just using a empty lambda `|| {}`.
+    fn run(&mut self, callback: impl Fn()) -> F::Result {
         self.base_mut().gen = 0;
         self.base_mut().time_start = Instant::now();
         self.init_pop();
@@ -307,7 +244,7 @@ pub trait Solver<F: ObjFunc>: Algorithm<F> {
             let b = self.base_mut();
             match b.task {
                 Task::MaxGen(v) => {
-                    b.pb.inc();
+                    callback();
                     if b.gen >= v {
                         break;
                     }
@@ -331,7 +268,6 @@ pub trait Solver<F: ObjFunc>: Algorithm<F> {
                 }
             }
         }
-        self.base().pb.finish();
         self.base_mut().report();
         self.base().func.result(&self.base().best)
     }
